﻿using System;

namespace ELTE.Basics.NumberTypes
{
    /// <summary>
    /// Komplex szám típusa
    /// </summary>
    class Complex : IRealCompatible
    {
        /// <summary>
        /// Valós rész lekérdezése vagy beállítása
        /// </summary>
        public Double Real { get; set; }
        /// <summary>
        /// Képzetes rész lekérdezése vagy beállítása
        /// </summary>
        public Double Imaginary { get; set; }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        public Complex()
        {
            Real = 0;
            Imaginary = 0;
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        /// <param name="real">valós rész</param>
        public Complex(Double real)
        {
            Real = real;
            Imaginary = 0;
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        /// <param name="real">valós rész</param>
        /// <param name="imaginary">képzetes rész</param>
        public Complex(Double real, Double imaginary)
        {
            Real = real;
            Imaginary = imaginary;
        }

        /// <summary>
        /// Összeadás komplex számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Complex a, Complex b)
        {
            return new Complex(a.Real + b.Real, a.Imaginary + b.Imaginary); 
        }

        /// <summary>
        /// Összeadás komplex és valós számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Complex a, Double b)
        {
            return new Complex(a.Real + b, a.Imaginary); 
        }

        /// <summary>
        /// Összeadás valós és komplex számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Double a, Complex b)
        {
            return new Complex(a + b.Real, b.Imaginary); 
        }

        /// <summary>
        /// Konjugálás
        /// </summary>
        /// <returns>Konjugált komplex szám</returns>
        public Complex Conjugation()
        {
            return new Complex(Real, -Imaginary);
        }

        /// <summary>
        /// Kiírás szövegként
        /// </summary>
        /// <returns>Komplex szám szövegszerű leírása</returns>
        public override string ToString()
        {
            if (Imaginary == 0)
                return Real.ToString();
            else if (Imaginary > 0)
                return Real.ToString() + "+" + Imaginary.ToString() + "i";
            else
                return Real.ToString() + "-" + Imaginary.ToString() + "i";
        }

        #region IRealCompatible Members

        public Double ToDouble()
        {
            return Real;
        }

        public void FromDouble(Double val)
        {
            Real = val;
            Imaginary = 0;
        }

        #endregion

        /// <summary>
        /// Explicit konverziós operátor.
        /// </summary>
        /// <param name="value">Komplex érték.</param>
        /// <returns>Lebegőpontos valós érték.</returns>
        public static explicit operator Double(Complex value)
        {
            return value.ToDouble();
        }
    }
}
