﻿using System;
using System.Collections.Generic;

namespace University
{
    /// <summary>
    /// Egyetemi polgár típus.
    /// </summary>
    abstract class Citizen
    {
        /// <summary>
        /// Foglalt azonosítók.
        /// </summary>
        protected static List<String> _takenCodes = new List<String>();

        /// <summary>
        /// Új, egyedi azonosító generálása.
        /// </summary>
        /// <param name="lastName">Vezetéknév.</param>
        /// <param name="firstName">Keresztnév.</param>
        /// <returns>Azonosító.</returns>
        protected static String GenerateCode(String lastName, String firstName)
        {
            Int32 counter = 1;
            Boolean match;
            String code;

            do
            {
                match = false;
                code = firstName.ToLower()[0] + lastName.ToLower() + counter;
                foreach (String takenCode in _takenCodes)
                {
                    if (code == takenCode)
                    {
                        match = true;
                        counter++;
                        break;
                    }
                }
            }
            while (match);
            return code;
        }

        /// <summary>
        /// A vezetéknév lekérdezése.
        /// </summary>
        /// <value>Vezetéknév.</value>
        public String LastName { get; protected set; }

        /// <summary>
        /// A keresztnév lekérdezése.
        /// </summary>
        /// <value>Keresztnév.</value>
        public String FirstName { get; protected set; }

        /// <summary>
        /// A teljes név lekérdezése.
        /// </summary>
        /// <value>Teljes név.</value>
        public String Name
        {
            get { return String.Format("{0} {1}", LastName, FirstName); }
        }

        /// <summary>
        /// Egyetemi azonosító lekérdezése.
        /// </summary>
        /// <value>Egyetemi azonosító.</value>
        public String Code { get; private set; }

        /// <summary>
        /// Új egyetemi polgár példányosítása.
        /// </summary>
        /// <param name="lastName">Vezetéknév.</param>
        /// <param name="firstName">Keresztnév.</param>
        public Citizen(String lastName, String firstName)
        {
            LastName = lastName;
            FirstName = firstName;
            Code = GenerateCode(LastName, FirstName);
            _takenCodes.Add(Code);
        }

        /// <summary>
        /// Új egyetemi polgár példányosítása.
        /// </summary>
        /// <param name="name">Teljes név.</param>
        /// <exception cref="System.ArgumentException">Amennyiben a név nem áll legalább 2 tagból;name</exception>
        public Citizen(String name)
        {
            String[] nameParts = name.Split(' ');
            if(nameParts.Length < 2)
                throw new ArgumentException("A névnek legalább 2 tagból kell állnia!", "name");

            LastName = nameParts[0]; 
            FirstName = nameParts[1];
            Code = GenerateCode(LastName, FirstName);
            _takenCodes.Add(Code);
        }

        public override String ToString()
        {
            return String.Format("Név: {0}, Azonosító: {1}", Name, Code);
        }
    }
}