using System;
using System.Collections.Generic;

namespace University
{
    /// <summary>
    /// Kurzus tpus.
    /// </summary>
    class Course
    {
        /// <summary>
        /// Kurzuslista.
        /// </summary>
        private static List<Course> _courses;

        /// <summary>
        /// A kurzusok lekrdezse.
        /// </summary>
        /// <value>Kurzuslista.</value>
        public static IList<Course> Courses
        {
            get { return _courses.AsReadOnly(); }
        }

        /// <summary>
        /// Statikus konstruktor.
        /// </summary>
        static Course()
        {
            _courses = new List<Course>();
        }

        /// <summary>
        /// A kurzus nevnek lekrdezse.
        /// </summary>
        /// <value>Kurzus neve.</value>
        public String Name { get; private set; }

        /// <summary>
        /// A kurzus oktatjnak lekrdezse.
        /// </summary>
        /// <value>Kurzus oktatja.</value>
        public Teacher Teacher { get; private set; }

        /// <summary>
        /// Hallgati lista.
        /// </summary>
        private List<Student> _students;

        /// <summary>
        /// A hallgatk listjnak lekrdezse.
        /// </summary>
        /// <value>Hallgati lista.</value>
        public List<Student> Students
        {
            get { return new List<Student>(_students); }
        }

        /// <summary>
        /// j kurzus pldnyostsa.
        /// </summary>
        /// <param name="name">A kurzus neve.</param>
        /// <param name="teacher">A kurzus oktatja.</param>
        public Course(String name, Teacher teacher)
        {
            Name = name;
            Teacher = teacher;
            _students = new List<Student>();
            _courses.Add(this);
        }

        /// <summary>
        /// Hallgat feliratkoztatsa a kurzusra.
        /// </summary>
        /// <param name="student">A hallgat.</param>
        /// <returns>A mvelet sikeressge.</returns>
        public Boolean AddStudent(Student student)
        {
            if (_students.Contains(student))
                return false;
            _students.Add(student);
            student.SignForCourse(this);
            return true;
        }

        public override String ToString()
        {
            return Name;
        }
    }
}