﻿using System;
using System.Collections.Generic;

namespace University
{
    /// <summary>
    /// Hallgató típus.
    /// </summary>
    class Student : Citizen
    {
        /// <summary>
        /// Hallgatói lista.
        /// </summary>
        private static List<Student> _students;

        /// <summary>
        /// A hallgatók listájának lekérdezése.
        /// </summary>
        /// <value>Hallgatói lista.</value>
        public static IList<Student> Students
        {
            get { return _students.AsReadOnly(); }
        }

        /// <summary>
        /// Statikus konstruktor.
        /// </summary>
        static Student()
        {
            _students = new List<Student>();
        }

        /// <summary>
        /// Kurzuslista.
        /// </summary>
        private List<Course> _courses;

        /// <summary>
        /// A hallgató kurzusainak lekérdezése.
        /// </summary>
        /// <value>Kurzuslista.</value>
        public List<Course> Courses
        {
            get { return new List<Course>(_courses); }
        }

        /// <summary>
        /// Új hallgató példányosítása.
        /// </summary>
        /// <param name="lastName">Vezetéknév.</param>
        /// <param name="firstName">Keresztnév.</param>
        public Student(String lastName, String firstName)
            : base(lastName, firstName)
        {
            _courses = new List<Course>();
            _students.Add(this);
        }

        /// <summary>
        /// Új hallgató példányosítása.
        /// </summary>
        /// <param name="name">Teljes név.</param>
        public Student(String name)
            : base(name)
        {
            _courses = new List<Course>();
            _students.Add(this);
        }

        /// <summary>
        /// A hallgató feliratkoztatása egy kurzusra.
        /// </summary>
        /// <param name="course">A kurzus.</param>
        /// <returns>A művelet sikeressége.</returns>
        public Boolean SignForCourse(Course course)
        {
            if (_courses.Contains(course))
                return false;
            _courses.Add(course);
            course.AddStudent(this);
            return true;
        }
    }
}