﻿using System;
using System.Collections.Generic;

namespace University
{
    /// <summary>
    /// Oktató típus.
    /// </summary>
    class Teacher : Citizen
    {
        /// <summary>
        /// Oktatói lista.
        /// </summary>
        private static List<Teacher> _teachers;

        /// <summary>
        /// Az oktatók listájának lekérdezése.
        /// </summary>
        /// <value>Oktatói lista.</value>
        public static IList<Teacher> Teachers
        {
            get { return _teachers.AsReadOnly(); }
        }

        /// <summary>
        /// Statikus konstruktor.
        /// </summary>
        static Teacher()
        {
            _teachers = new List<Teacher>();
        }

        /// <summary>
        /// Új oktató példányosítása.
        /// </summary>
        /// <param name="lastName">Vezetéknév.</param>
        /// <param name="firstName">Keresztnév.</param>
        public Teacher(String lastName, String firstName)
            : base(lastName, firstName)
        {
            _teachers.Add(this);
        }

        /// <summary>
        /// Új oktató példányosítása.
        /// </summary>
        /// <param name="name">Teljes név.</param>
        public Teacher(String name)
            : base(name)
        {
            _teachers.Add(this);
        }

        /// <summary>
        /// Az oktatóhoz rendelt kurzus létrehozása.
        /// </summary>
        /// <param name="name">A létrehozandó kurzus neve.</param>
        /// <returns>A kurzus.</returns>
        public Course CreateCourse(String name)
        {
           return new Course(name, this);
        }

        /// <summary>
        /// Kurzus létrehozása.
        /// </summary>
        /// <param name="courseName">A létrehozandó kurzus neve.</param>
        /// <param name="teacher">A kurzus oktatója.</param>
        /// <returns>A kurzus.</returns>
        public static Course operator +(String courseName, Teacher teacher)
        {
            return teacher.CreateCourse(courseName);
        }
    }
}
