﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ThesisGenerator.Test
{
    [TestClass]
    public class ModelTest
    {
        [TestMethod]
        public void AttributeTest()
        {
            Model model = new Model(10, 5);
            Assert.AreEqual(model.ThesisCount, 10);
            Assert.AreEqual(model.RoomSize, 5);

            model.Next();
            Assert.AreEqual(10, model.ThesisCount);
            Assert.AreEqual(5, model.RoomSize);
        }

        [TestMethod]
        public void LastTakenTest()
        {
            Model model = new Model(10, 5);
            Int32 taken = model.Next();
            Assert.AreEqual(taken, model.LastTaken);
            taken = model.Next();
            Assert.AreEqual(taken, model.LastTaken);
        }

        [TestMethod]
        public void TakenTest()
        {
            Model model = new Model(10, 5);
            Int32[] taken = new Int32[4];
            for (Int32 i = 0; i < taken.Length; ++i)
                taken[i] = model.Next();
            for (Int32 i = 0; i < taken.Length; ++i)
                Assert.IsTrue(model.IsTaken(taken[i]));

            model.Next();
            Assert.AreEqual(model.IsTaken(taken[0]), model.LastTaken == taken[0]);
        }

        [TestMethod]
        public void JustEnoughThesisTest()
        {
            Model model = new Model(10, 10);
            Int32[] taken1 = new Int32[10];
            for (Int32 i = 0; i < taken1.Length; ++i)
                taken1[i] = model.Next();

            Int32[] taken2 = new Int32[10];
            for (Int32 i = 0; i < taken2.Length; ++i)
                taken2[i] = model.Next();

            Assert.IsTrue(Enumerable.SequenceEqual(taken1, taken2));
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void ZeroThesisTest()
        {
            Model model = new Model(0, 5);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void NegativeThesisTest()
        {
            Model model = new Model(-1, 5);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void NotEnoughThesisTest()
        {
            Model model = new Model(10, 11);
        }
    }
}
