﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ThesisGenerator
{
    public class Model
    {
        private HashSet<Int32> _elements;
        private Queue<Int32> _taken;
        private Random _generator = new Random();

        public Int32 ThesisCount { get; private set; }
        public Int32 RoomSize { get; private set; }
        public Int32 LastTaken { get; private set; }

        public Model(Int32 thesisCount, Int32 roomSize)
        {
            if(thesisCount <= 0)
                throw new ArgumentException("At least 1 thesis must exist!", "thesisCount");
            if(roomSize > thesisCount)
                throw new ArgumentException("Not enough theses!", "roomSize");

            ThesisCount = thesisCount;
            RoomSize = roomSize;

            _elements = new HashSet<Int32>();
            for (Int32 i = 1; i <= thesisCount; ++i)
                _elements.Add(i);
            _taken = new Queue<Int32>();
        }

        public Int32 Next()
        {
            LastTaken = _elements.ElementAt(_generator.Next(_elements.Count));
            _elements.Remove(LastTaken);
            _taken.Enqueue(LastTaken);
            if (_taken.Count >= RoomSize)
                _elements.Add(_taken.Dequeue());
            return LastTaken;
        }

        public Boolean IsTaken(Int32 thesis)
        {
            return _taken.Contains(thesis);
        }
    }
}
